/**
 * @file    stm8l15x_it.c
 * @brief   ELV-SH-IRS8 interrupt handlers
 * @author  ELV Elektronik AG
**/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2021-2022 ELV Elektronik AG - info@elv.com
 * 
 * This file is part of ELV-SH-IRS8 Firmware.
 * ELV-SH-IRS8 Firmware is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ELV-SH-IRS8 Firmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with ELV-SH-IRS8 Firmware. If not, see <https://www.gnu.org/licenses/>. 
 *---------------------------------------------------------------------------*/

/*
 *  Includes ------------------------------------------------------------------
 */
#include "defines.h"
#include "timer.h"
#include "irmp.h"
#include "irsnd.h"

/*
 *  Prototypes ----------------------------------------------------------------
 */
__interrupt void TIM2_UPD_OVF_TRG_BRK_USART2_TX_IRQ_Handler( void );
__interrupt void TIM3_UPD_OVF_TRG_BRK_USART3_TX_IRQ_Handler( void );
__interrupt void TIM1_UPD_OVF_TRG_COM_IRQ_Handler( void );
__interrupt void TIM1_CC_IRQ_Handler( void );
__interrupt void TIM4_UPD_OVF_TRG_IRQ_Handler( void );

/*
 *  Functions -----------------------------------------------------------------
 */

/*
  * @brief TIM2 Update/Overflow/Trigger/Break /USART2 TX Interrupt Handler
  * @param  None
  * @retval None
  */
#pragma vector = 21 
__interrupt void TIM2_UPD_OVF_TRG_BRK_USART2_TX_IRQ_Handler( void )
{
  TIM2_SR1 &= (uint8_t)(~TIM_SR1_UIF);
  // if TSOP receiver also detects my own IR LED, don't receive while sending:
  if (!irsnd_ISR())                  // call irsnd ISR
  {                                   // if not busy...
    irmp_ISR();                     // call irmp ISR
  }
}

/**
  * @brief Timer3 Update/Overflow/Trigger/Break Interrupt Handler
  * @param  None
  * @retval None
  */
#pragma vector = 23 
__interrupt void TIM3_UPD_OVF_TRG_BRK_USART3_TX_IRQ_Handler( void )
{
  TIM3_SR1 &= (uint8_t)(~TIM_SR1_UIF);
}

/**
  * @brief TIM1 Update/Overflow/Trigger/Commutation Interrupt Handler
  * @param  None
  * @retval None
  */
#pragma vector = 25 
__interrupt void TIM1_UPD_OVF_TRG_COM_IRQ_Handler( void )
{
  TIM1_SR1 &= (uint8_t)(~TIM_SR1_UIF);
}

/**
  * @brief TIM1 Capture/Compare Interrupt Handler
  * @param  None
  * @retval None
  */
#pragma vector = 26 
__interrupt void TIM1_CC_IRQ_Handler( void )
{
  /* TIM1 channel 2 toggling */
  if (TIM1_SR1 & (uint8_t)TIM_SR1_CC2IF && TIM1_IER & (uint8_t)TIM_IER_CC2IE)
  {
    TIM1_SR1 &= (uint8_t)(~TIM_SR1_CC2IF);
  }
  /* TIM1 channel 3 toggling */
  if (TIM1_SR1 & (uint8_t)TIM_SR1_CC3IF && TIM1_IER & (uint8_t)TIM_IER_CC3IE)
  {
    TIM1_SR1 &= (uint8_t)(~TIM_SR1_CC3IF);
  }
}

/**
  * @brief TIM4 Update/Overflow/Trigger Interrupt Handler
  * @param  None
  * @retval None
  */
#pragma vector = 27 
__interrupt void TIM4_UPD_OVF_TRG_IRQ_Handler( void )
{
  TIM4_SR1 &= (uint8_t)(~TIM_SR1_UIF);
  TIMER_do_timer();
}

/**
  * @}
  */ 
