/**
 * @file    defines.h
 * @brief   This file contains some definitions for the project ELV-SH-IRS8
 * @author  ELV Elektronik AG
**/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2021-2022 ELV Elektronik AG - info@elv.com
 * 
 * This file is part of ELV-SH-IRS8 Firmware.
 * ELV-SH-IRS8 Firmware is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ELV-SH-IRS8 Firmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with ELV-SH-IRS8 Firmware. If not, see <https://www.gnu.org/licenses/>. 
 *---------------------------------------------------------------------------*/
#ifndef __DEFINE_H__
#define __DEFINE_H__

#define DEBUG_OUTPUT 1
#if (DEBUG_OUTPUT == 0)
  #define DEBUG_message(...)
#elif (DEBUG_OUTPUT == 1)
  #define DEBUG_message(...) printf (__VA_ARGS__)
#endif

/**
 * Includes section ----------------------------------------------------------------------
**/
#include <libstm8/stm8l.h>
#include <libstm8/l/gpio.h>
#include "typedefs_stm8.h"
/**
 * Defines and constants section ---------------------------------------------------------
**/
/* EEPROM storage */
#define MAX_STORAGE_PER_CH 10 // 1 to 10 allowed

/* version output */
#define VERSION_PROGRAM_TYPE      APPLICATION
#define VERSION_MAJOR_VERSION     1
#define VERSION_MINOR_VERSION     0
#define VERSION_BUILD_VERSION     4
           
/* USART */
#define USART_PORT                GPIOC
#define USART_RX_PIN              GPIO2
#define USART_TX_PIN              GPIO3

#define IR_PORT                   GPIOB //define IRMP_PORT_LETTER in irmpconfig.h
#define IR_PIN                    GPIO3 //define IRMP_BIT_NUMBER in irmpconfig.hGPIO
#define IROUT_PORT                GPIOD
#define IROUT_PIN                 GPIO2
#define IREXT_PORT                GPIOD
#define IREXT_PIN                 GPIO4

#define S1_PORT                  GPIOE
#define S1_PIN                   GPIO0
#define S2_PORT                  GPIOE
#define S2_PIN                   GPIO1
#define S3_PORT                  GPIOE
#define S3_PIN                   GPIO2
#define S4_PORT                  GPIOE
#define S4_PIN                   GPIO3
#define S5_PORT                  GPIOE
#define S5_PIN                   GPIO4
#define S6_PORT                  GPIOE
#define S6_PIN                   GPIO5
#define S7_PORT                  GPIOE
#define S7_PIN                   GPIO6
#define S8_PORT                  GPIOE
#define S8_PIN                   GPIO7
#define S9_PORT                  GPIOA
#define S9_PIN                   GPIO5
#define S10_PORT                 GPIOA
#define S10_PIN                  GPIO4
#define OUT1_PORT                GPIOB
#define OUT1_PIN                 GPIO4
#define OUT2_PORT                GPIOB
#define OUT2_PIN                 GPIO5
#define OUT3_PORT                GPIOB
#define OUT3_PIN                 GPIO6
#define OUT4_PORT                GPIOB
#define OUT4_PIN                 GPIO7
#define OUT5_PORT                GPIOC
#define OUT5_PIN                 GPIO4
#define OUT6_PORT                GPIOC
#define OUT6_PIN                 GPIO5
#define OUT7_PORT                GPIOC
#define OUT7_PIN                 GPIO6
#define OUT8_PORT                GPIOC
#define OUT8_PIN                 GPIO7
#define T1_PORT                  GPIOA
#define T1_PIN                   GPIO6
#define T2_PORT                  GPIOA
#define T2_PIN                   GPIO7
#define T3_PORT                  GPIOD
#define T3_PIN                   GPIO7
#define T4_PORT                  GPIOD
#define T4_PIN                   GPIO6
#define T5_PORT                  GPIOD
#define T5_PIN                   GPIO5
#define T6_PORT                  GPIOD
#define T6_PIN                   GPIO3


#define LED_RED_PORT              GPIOB
#define LED_RED_PIN               GPIO0
#define LED_GREEN_PORT            GPIOB
#define LED_GREEN_PIN             GPIO2
#define LED_S9_PORT               GPIOB
#define LED_S9_PIN                GPIO1
#define LED_S10_PORT              GPIOD
#define LED_S10_PIN               GPIO0

/* Key definitions */
/* KEY.C */
enum
{
  S1 = 0,
  S2,
  S3,
  S4,
  S5,
  S6,
  S7,
  S8,
  S9,
  S10,
  OUT1,
  OUT2,
  OUT3,
  OUT4,
  OUT5,
  OUT6,
  OUT7,
  OUT8,
//  T1,
//  T2,
//  T3,
//  T4,
//  T5,
//  T6,
  KEY_TOTAL_NUMBER
};

/* TIMER.C */
#define TIM4_PRESCALER            0x07
#define TIM4_PERIOD_1MS           124


#endif /* __define_H__ */
/** @}
 * End of file
**/
