/*--------------------------------------------------------------------------*/
/**
 * @file    config.c
 * @brief   config routines
 * @author  ELV Elektronik AG
**/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2021-2022 ELV Elektronik AG - info@elv.com
 * 
 * This file is part of ELV-SH-IRS8 Firmware.
 * ELV-SH-IRS8 Firmware is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ELV-SH-IRS8 Firmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with ELV-SH-IRS8 Firmware. If not, see <https://www.gnu.org/licenses/>. 
 *---------------------------------------------------------------------------*/

/*
 * Includes section -----------------------------------------------------------
*/
#include "typedefs_stm8.h"
#include <libstm8/stm8l.h>
#include <libstm8/l/clk.h>
#include <libstm8/l/gpio.h>
#include <libstm8/l/tim.h>
#include "defines.h"
#include "config.h"

/*
 *  Functions -----------------------------------------------------------------
 */
/**
 * @brief Configures the system clock
 * @note use 16MHz main-clock
 * @param none 
 * @return none
**/
void CONFIG_clock( void )
{
  //Clock deinit
  CLK_DIVR = 0x03;
  CLK_CRTCR = 0x00;
  CLK_ICKCR = 0x11;
  CLK_PCKENR1 = 0x00;
  CLK_PCKENR2 = 0x80;
  CLK_CCOR = 0x00;
  CLK_ECKCR = 0x00;
  CLK_SWR  = 0x01;
  CLK_SWCR = 0x00;
  CLK_CSSR  = 0x00;
  CLK_CBEEPR = 0x00;
//  CLK_HSITRIMR = 0x00;
//  CLK_HSIUNLCKR = 0x00;
  CLK_REGCSR = 0xB9;
  CLK_PCKENR3 = 0x00;

  /* Set HSEON bit */
  CLK_ICKCR |= CLK_ICK_HSION;
    
  // Set up the HSE clock
  // Set systemclock back to 16 MHz
  CLK_DIVR = CLK_DIV_CKM_1;
  // Enable clock switching
  /* Set SWEN bit */
  CLK_SWCR |= CLK_SWC_EN;
    // Set systemclock source
  /* Selection of the target clock source */
  CLK_SWR = CLK_CRTC_SEL_HSE;
  
  // Check if HSE clock is system clock
  while( CLK_SCSR != CLK_SW_HSE )
  {
    // ToDo: Abort condition??
  };
}

/**
 * @brief Configures the GPIO peripheral.
 * @param none 
 * @return none
**/ 
void CONFIG_gpio(void)
{
   // erase all settings on the GPIO pins
  GPIOA_CR2 = GPIO_CR2_ID; 
  GPIOA_ODR = 0x00;
  GPIOA_DDR = GPIO_DDR_INPUT;
  GPIOA_CR1 = GPIO_CR1_FI;
  GPIOB_CR2 = GPIO_CR2_ID; 
  GPIOB_ODR = 0x00;
  GPIOB_DDR = GPIO_DDR_INPUT;
  GPIOB_CR1 = GPIO_CR1_FI;
  GPIOC_CR2 = GPIO_CR2_ID; 
  GPIOC_ODR = 0x00;
  GPIOC_DDR = GPIO_DDR_INPUT;
  GPIOC_CR1 = GPIO_CR1_FI;
  GPIOD_CR2 = GPIO_CR2_ID; 
  GPIOD_ODR = 0x00;
  GPIOD_DDR = GPIO_DDR_INPUT;
  GPIOD_CR1 = GPIO_CR1_FI;
  GPIOE_CR2 = GPIO_CR2_ID; 
  GPIOE_ODR = 0x00;
  GPIOE_DDR = GPIO_DDR_INPUT;
  GPIOE_CR1 = GPIO_CR1_FI;
  GPIOF_CR2 = GPIO_CR2_ID; 
  GPIOF_ODR = 0x00;
  GPIOF_DDR = GPIO_DDR_INPUT;
  GPIOF_CR1 = GPIO_CR1_FI;;

  gpio_mode_setup(IR_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,IR_PIN); 

  gpio_mode_setup (IROUT_PORT,GPIO_MODE_OUTPUT, GPIO_CR1_PP, GPIO_CR2_10,IROUT_PIN); 
  GPIO_ODR(IROUT_PORT) &= (uint8_t)(~(IROUT_PIN));
  
  gpio_mode_setup (IREXT_PORT,GPIO_MODE_OUTPUT, GPIO_CR1_PP, GPIO_CR2_10,IREXT_PIN); 
  GPIO_ODR(IREXT_PORT) &= (uint8_t)(~(IREXT_PIN));
  
  gpio_mode_setup (S1_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,S1_PIN); 
  gpio_mode_setup (S2_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,S2_PIN); 
  gpio_mode_setup (S3_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,S3_PIN); 
  gpio_mode_setup (S4_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,S4_PIN); 
  gpio_mode_setup (S5_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,S5_PIN); 
  gpio_mode_setup (S6_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,S6_PIN); 
  gpio_mode_setup (S7_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,S7_PIN); 
  gpio_mode_setup (S8_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,S8_PIN); 
  gpio_mode_setup (S9_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,S9_PIN); 
  gpio_mode_setup (S10_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,S10_PIN); 
  gpio_mode_setup (OUT1_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,OUT1_PIN); 
  gpio_mode_setup (OUT2_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,OUT2_PIN); 
  gpio_mode_setup (OUT3_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,OUT3_PIN); 
  gpio_mode_setup (OUT4_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,OUT4_PIN); 
  gpio_mode_setup (OUT5_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,OUT5_PIN); 
  gpio_mode_setup (OUT6_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,OUT6_PIN); 
  gpio_mode_setup (OUT7_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,OUT7_PIN); 
  gpio_mode_setup (OUT8_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,OUT8_PIN); 
  gpio_mode_setup (T1_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,T1_PIN); 
  gpio_mode_setup (T2_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,T2_PIN); 
  gpio_mode_setup (T3_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,T3_PIN); 
  gpio_mode_setup (T4_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,T4_PIN); 
  gpio_mode_setup (T5_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,T5_PIN); 
  gpio_mode_setup (T6_PORT,GPIO_MODE_INPUT, GPIO_CR1_PU, GPIO_CR2_ID,T6_PIN); 
  
  // LED
  gpio_mode_setup (LED_RED_PORT,GPIO_MODE_OUTPUT, GPIO_CR1_PP, GPIO_CR2_10,LED_RED_PIN); 
  GPIO_ODR(LED_RED_PORT) &= (uint8_t)(~(LED_RED_PIN));
  gpio_mode_setup (LED_GREEN_PORT,GPIO_MODE_OUTPUT, GPIO_CR1_PP, GPIO_CR2_10,LED_GREEN_PIN); 
  GPIO_ODR(LED_GREEN_PORT) &= (uint8_t)(~(LED_GREEN_PIN));
  
  gpio_mode_setup (LED_S9_PORT,GPIO_MODE_OUTPUT, GPIO_CR1_PP, GPIO_CR2_10,LED_S9_PIN); 
  GPIO_ODR(LED_S9_PORT) &= (uint8_t)(~(LED_S9_PIN));
  gpio_mode_setup (LED_S10_PORT,GPIO_MODE_OUTPUT, GPIO_CR1_PP, GPIO_CR2_10,LED_S10_PIN); 
  GPIO_ODR(LED_S10_PORT) &= (uint8_t)(~(LED_S10_PIN));
  
  gpio_mode_setup (GPIOC,GPIO_MODE_OUTPUT, GPIO_CR1_PP, GPIO_CR2_10,GPIO0); 
  GPIO_ODR(GPIOC) &= (uint8_t)(~(GPIO0));
  gpio_mode_setup (GPIOC,GPIO_MODE_OUTPUT, GPIO_CR1_PP, GPIO_CR2_10,GPIO1); 
  GPIO_ODR(GPIOC) &= (uint8_t)(~(GPIO1));
  gpio_mode_setup (GPIOD,GPIO_MODE_OUTPUT, GPIO_CR1_PP, GPIO_CR2_10,GPIO1); 
  GPIO_ODR(GPIOD) &= (uint8_t)(~(GPIO1));
  gpio_mode_setup (GPIOF,GPIO_MODE_OUTPUT, GPIO_CR1_PP, GPIO_CR2_10,GPIO0); 
  GPIO_ODR(GPIOF) &= (uint8_t)(~(GPIO0));
  
}

/**
 * @brief Configures the USART peripheral.
 * @param none 
 * @return none
**/ 
void CONFIG_usart( void )
{
  /* Clear the Idle Line Detected bit in the status register by a read
  to the USART_SR register followed by a Read to the USART_DR register */
  (void) USART1_SR;
  (void) USART1_DR;
  
  USART1_BRR2 = 0x00;
  USART1_BRR1 = 0x00;
  USART1_CR1 = 0x00; 
  USART1_CR2 = 0x00; 
  USART1_CR3 = 0x00; 
  USART1_CR4 = 0x00; 
  
  clk_periph_clock_enable(CLK_USART1);
  GPIOC_CR1 |= GPIO2;
  GPIOC_CR1 |= GPIO3;
  
  /* Configure USART peripheral */
  usart_set_databits(USART1, 8);
  usart_set_parity(USART1, USART_PARITY_NONE);
  usart_set_stopbits(USART1, USART_STOPBITS_1);
  usart_set_baudrate(USART1, (uint32_t)115200);
  usart_enable_mode(USART1, USART_MODE_TX);
}

/**
 * @brief Configures the TIMER4 peripheral.
 * @note Set TIM to generate 1ms timings
 * @param none 
 * @return none
**/
void CONFIG_timer(void)
{
  /* Enable TIM4 CLK */
  clk_periph_clock_enable(CLK_TIM4);
  /* Time base configuration */
  /* TIM4 period is TIM4_PRESCALER * (TIM4_PERIOD + 1) / TIM4_CLK 
                              = 64 * 125 / 8.000.000 = 1 ms */
  timer_set_prescaler(TIM4_BASE, TIM4_PRESCALER);
  timer_set_period(TIM4_BASE, TIM4_PERIOD_1MS);
  /* Generate an update event to reload the Prescaler value immediately */
  TIM4_EGR = 0x01;
  
  /* Clear TIM4 update flag */
  TIM4_SR1 = (uint8_t)(~TIM_SR1_UIF);
  /* Enable update interrupt */
  TIM4_IER |= TIM_IER_UIE;
  /* Enable TIM4 */
  TIM4_CR1 |= TIM_CR1_CEN ;
}
/** @}
 * End of file
**/
