/**
 * @file    keys.h
 * @brief   This file contains functionprototypes for the key-management-routines
 * @author  ELV Elektronik AG
**/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2021-2022 ELV Elektronik AG - info@elv.com
 * 
 * This file is part of ELV-SH-IRS8 Firmware.
 * ELV-SH-IRS8 Firmware is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ELV-SH-IRS8 Firmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with ELV-SH-IRS8 Firmware. If not, see <https://www.gnu.org/licenses/>. 
 *---------------------------------------------------------------------------*/

/*
 * Defines --------------------------------------------------------------------
 */
#ifndef __KEY_H__
#define __KEY_H__

/*
 *  Includes ------------------------------------------------------------------
 */
#include "defines.h"

/*
 *  Definitions ---------------------------------------------------------------
 */
#define KEY_DEBOUNCE_SAMPLES  7

/*
 *  Prototypes ----------------------------------------------------------------
 */
extern void KEY_do_keys( void );

extern bool_t KEY_is_key_pressed(uint8_t u8_key);
extern bool_t KEY_is_key_just_pressed(uint8_t u8_key);
extern bool_t KEY_is_key_pressed_short(uint8_t Key, uint8_t Maximum_Time);
extern bool_t KEY_is_key_pressed_for_minimum_time(uint8_t u8_key, uint8_t u8_min_time);
extern bool_t KEY_is_key_pressed_for_minimum_time_with_repeat( uint8_t u8_key, 
                                                               uint8_t u8_minimum_time, 
                                                               uint8_t u8_delay_time );
extern bool_t KEY_is_key_pressed_long(uint8_t Key, uint8_t Minimum_Time);

extern bool_t KEY_is_key_just_released(uint8_t u8_key);
extern bool_t KEY_is_key_released_short(uint8_t u8_key, uint8_t u8_maximum_time);
extern bool_t KEY_is_key_released_for_minimum_time(uint8_t u8_key, uint8_t u8_min_time);
extern bool_t KEY_is_key_released_long(uint8_t u8_key, uint8_t u8_time);
extern bool_t KEY_was_key_released_before_time(uint8_t u8_key, uint8_t u8_time);
extern bool_t KEY_was_key_released_after_time(uint8_t u8_key, uint8_t u8_time);

extern uint8_t KEY_get_key_status_duration( uint8_t u8_key );


#endif /* __KEY_H__ */

/** @} */
/** @} */
/** @}
 * End of file
**/
