/**
 * @file    my_functions.c
 * @brief   function collection for IR, LED, TIMER and EEPROM handling
 * @author  ELV Elektronik AG
**/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2021-2022 ELV Elektronik AG - info@elv.com
 * 
 * This file is part of ELV-SH-IRS8 Firmware.
 * ELV-SH-IRS8 Firmware is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ELV-SH-IRS8 Firmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with ELV-SH-IRS8 Firmware. If not, see <https://www.gnu.org/licenses/>. 
 *---------------------------------------------------------------------------*/

/*
 *  Includes ------------------------------------------------------------------
 */
#include <stdio.h>
#include "defines.h"
#include "irmp.h"
#include "irsnd.h"
#include "my_functions.h"

/*
 *  Prototypes ----------------------------------------------------------------
 */
/******************************************************************************
 *  MY_FUNCTIONS_IR prototypes
 *  this section containt the prototypes for the ir handling
******************************************************************************/
void MY_FUNCTIONS_IR_init( void );
void MY_FUNCTIONS_IR_timer( void );
void MY_FUNCTIONS_IR_do( void );
void MY_FUNCTIONS_IR_send( uint8_t channel );
IR_state_t MY_FUNCTIONS_IR_get_state( void );
uint8_t MY_FUNCTIONS_IR_get_act_channel( void );
void MY_FUNCTIONS_IR_stop_learning( void );
void MY_FUNCTIONS_IR_start_learning( uint8_t channel );

/******************************************************************************
 *  MY_FUNCTIONS_LED prototypes
 *  this sections containt the prototypes for duo color device led
******************************************************************************/
void MY_FUNCTIONS_LED_do( void );
led_state_t MY_FUNCTIONS_LED_get_state( void );
void MY_FUNCTIONS_LED_set(led_state_t state, bool_t immediatly);

/******************************************************************************
 *  MY_FUNCTIOS_TIMER prototypes
 *  this sections containt the prototypes for timer channels
******************************************************************************/
void MY_FUNCTIONS_TIMER_do( void );
void MY_FUNCTIONS_TIMER_isr( void );
void MY_FUNCTIONS_TIMER_tim1_set_laststate( bool_t new_state );
void MY_FUNCTIONS_TIMER_tim2_set_laststate( bool_t new_state );
bool_t MY_FUNCTIONS_TIMER_tim1_get_laststate( void );
bool_t MY_FUNCTIONS_TIMER_tim2_get_laststate( void );
void MY_FUNCTIONS_TIMER_tim1_restart( void );
void MY_FUNCTIONS_TIMER_tim2_restart( void );
void MY_FUNCTIONS_TIMER_tim1_stop( void );
void MY_FUNCTIONS_TIMER_tim2_stop( void );
bool_t MY_FUNCTIONS_TIMER_tim1_is_active( void );
bool_t MY_FUNCTIONS_TIMER_tim2_is_active( void );
void MY_FUNCTIONS_TIMER_LEDS_init( void );
void MY_FUNCTIONS_TIMER_LEDS_fading( void );
/******************************************************************************
 *  MY_FUNCTIOS_EEPROM prototypes
 *  this sections containt the prototypes for data storage in eeprom
******************************************************************************/
void MY_FUNCTIONS_EEPROM_init( void );
void MY_FUNCTIONS_EEPROM_get_ir(uint8_t channel, uint8_t index, IRMP_DATA* data, uint16_t* time);
void MY_FUNCTIONS_EEPROM_save_ir(uint8_t channel, uint8_t index, IRMP_DATA data, uint16_t time);
IRMP_DATA MY_FUNCTIONS_EEPROM_get_irmp_code( uint8_t index );
void MY_FUNCTIONS_EEPROM_set_irmp_code( IRMP_DATA new_value, uint8_t index );
uint16_t MY_FUNCTIONS_EEPROM_get_ir_time( uint8_t index );
void MY_FUNCTIONS_EEPROM_set_ir_time( uint16_t new_value, uint8_t index );
uint32_t MY_FUNCTIONS_EEPROM_get_init_value( void );
void MY_FUNCTIONS_EEPROM_set_init_value( uint32_t u32_new_init_value );
uint32_t MY_FUNCTIONS_EEPROM_get_bootloader_value( void );
void MY_FUNCTIONS_EEPROM_set_bootloader_value( uint32_t u32_new_bootloader_value );

/*
 *  Definitions ---------------------------------------------------------------
 */
#define CONF1                   ((GPIO_IDR(T1_PORT) & T1_PIN) == 0)
#define CONF2                   ((GPIO_IDR(T2_PORT) & T2_PIN) == 0)
#define CONF3                   ((GPIO_IDR(T3_PORT) & T3_PIN) == 0)
#define CONF4                   ((GPIO_IDR(T4_PORT) & T4_PIN) == 0)
#define CONF5                   ((GPIO_IDR(T5_PORT) & T5_PIN) == 0)
#define CONF6                   ((GPIO_IDR(T6_PORT) & T6_PIN) == 0)
#define GET_TIM1_STOP_CONF            (CONF1|(CONF2<<1)|(CONF3<<2))
#define GET_TIM2_STOP_CONF            (CONF4|(CONF5<<1)|(CONF6<<2))

/*
 *  Variables ------------------------------------------------------------------
 */
/******************************************************************************
 *  MY_FUNCTIONS_IR variables
 *  this section containt the variables for the ir handling
******************************************************************************/
IRMP_DATA irmp_data;
IRMP_DATA irmp_data_mem[MAX_STORAGE_PER_CH];
uint16_t u16_time_stored[MAX_STORAGE_PER_CH];

IR_state_t ir_state;
uint8_t u8_ir_act_channel;
uint8_t ir_index_mem = 0;
uint8_t ir_repeat_counter;

IRMP_DATA* data = &irmp_data;
IRMP_DATA* ir_data_mem = &irmp_data_mem[0];
uint16_t*  ir_time_mem = &u16_time_stored[0];
uint16_t u16_ir_timer = 0xffff;


/******************************************************************************
 *  MY_FUNCTIONS_LED variables
 *  this section containt the variables for duo color device led
******************************************************************************/
led_state_t led_state = LED_OFF;
uint8_t u8_led_time = 0;

static const uint8_t FADING_TABLE[254] = {2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 7, 7, 7, 7, 7, 8, 8, 8, 8, 9, 9, 9, 9, 10, 10, 10, 11, 11, 11, 12, 12, 12, 13, 13, 14, 14, 15, 15, 16, 16, 17, 17, 18, 18, 19, 19, 20, 21, 21, 22, 23, 24, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 38, 39, 40, 42, 43, 45, 46, 48, 49, 51, 53, 54, 56, 58, 60, 62, 64, 67, 69, 71, 74, 76, 79, 79, 76, 74, 71, 69, 67, 64, 62, 60, 58, 56, 54, 53, 51, 49, 48, 46, 45, 43, 42, 40, 39, 38, 36, 35, 34, 33, 32, 31, 30, 29, 28, 27, 26, 25, 24, 24, 23, 22, 21, 21, 20, 19, 19, 18, 18, 17, 17, 16, 16, 15, 15, 14, 14, 13, 13, 12, 12, 12, 11, 11, 11, 10, 10, 10, 9, 9, 9, 9, 8, 8, 8, 8, 7, 7, 7, 7, 7, 6, 6, 6, 6, 6, 6, 5, 5, 5, 5, 5, 5, 5, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2}; 
static const unsigned char CONFIG_LOOKUP[] = { 0, 4, 6, 5, 8, 7, 9, 10 };
TIMER_t led_timer1;
TIMER_t led_timer2;

/******************************************************************************
 *  MY_FUNCTIONS_EEPROM variables
 *  this section containt the variables for data storage in eeprom
******************************************************************************/
__no_init __eeprom IRMP_DATA ee_irmp_code[12*MAX_STORAGE_PER_CH] @ 0x1000;
__no_init __eeprom uint16_t ee_irmp_time[12*MAX_STORAGE_PER_CH] @ (0x1000+(sizeof(IRMP_DATA)*12*MAX_STORAGE_PER_CH));
__no_init __eeprom uint32_t ee_u32_init @ 0x17fc;

/*
 *  Prototypes ----------------------------------------------------------------
 */

/*
 *  Functions -----------------------------------------------------------------
 */
/******************************************************************************
 *  MY_FUNCTIONS_IR functions
 *  this section containt the functions for the ir handling
******************************************************************************/
/* @brief Init function for IR receive and send
 *
 * @param none
 * @return none
**/
void MY_FUNCTIONS_IR_init( void )
{
  irmp_init();
  irsnd_init(); // initialize irmp
  
  /* Enable TIM2 CLK */
  clk_periph_clock_enable(CLK_TIM2);

  /* Timer 2 configuration */
  timer_set_mode (TIM2_BASE, TIM_CMS_EDGE,TIM_DIR_UP);
  timer_set_prescaler(TIM2_BASE, 0x00);
  timer_set_period(TIM2_BASE, ((F_CPU / F_INTERRUPTS)/1) - 1);
  TIM2_EGR = 0x01;
  /* Clear TIM2 update flag */
  TIM2_SR1 = (uint8_t)(~TIM_SR1_UIF);

  /* Enable update interrupt */
  TIM2_IER |= TIM_IER_UIE;
  /* Enable TIM2 */
  timer_enable_counter(TIM2_BASE);
}


/* @brief do function for ir handling
 *
 * @param none 
 * @return none
**/
void MY_FUNCTIONS_IR_do( void )
{
  if( ir_state == IR_SEND )
  {
     if(u16_ir_timer == 0)
     {
        MY_FUNCTIONS_LED_set(LED_PULSE_RED, __TRUE);
        MY_FUNCTIONS_EEPROM_get_ir(u8_ir_act_channel, ir_index_mem, data, &u16_ir_timer);
        DEBUG_message( "IR-Send: ");
        DEBUG_message( "%u, ", data->protocol );
        DEBUG_message( "%u, ", data->address );
        DEBUG_message( "%u, ", data->command );
        DEBUG_message( "%u, ", data->flags );
        DEBUG_message( "%u, ", u16_ir_timer );
        DEBUG_message( "\r\n" );
        if( irsnd_send_data(data, TRUE) )
        {
          //code accepted and start sending
          if(u16_ir_timer == U16_TIMER_DISABLE)
          {
            MY_FUNCTIONS_LED_set(LED_OFF, __FALSE);
            ir_state = LEARNING_OFF;
          }
          else
          {
            //blink red
            MY_FUNCTIONS_LED_set(LED_PULSE_OFF_RED, __FALSE);
           }
        }
        else
        {
          if(u16_ir_timer == U16_TIMER_DISABLE)
          {
            MY_FUNCTIONS_LED_set(LED_OFF, __TRUE);
            ir_state = LEARNING_OFF;
          }
          else
          {
            //wrong code step over to next one
            u16_ir_timer = 0;
          }
        }
        
        ir_index_mem++; //select next command
        if(ir_index_mem >= MAX_STORAGE_PER_CH) //stop if index exceeds limit
        {
          MY_FUNCTIONS_LED_set(LED_OFF, __FALSE);
          ir_state = LEARNING_OFF;
        }
     }
  }
  else if(ir_state == LEARNING_ON)
  {
    if(MY_FUNCTIONS_LED_get_state() == LED_OFF)
    {
      //restart blinking orange
      MY_FUNCTIONS_LED_set(LED_PULSE_ORANGE, __FALSE);
    }
    if( irmp_get_data (data))
    {
      //blink green
      MY_FUNCTIONS_LED_set(LED_PULSE_GREEN, __TRUE);
      DEBUG_message( "IR-Received: ");
      DEBUG_message( "%u, ", data->protocol );
      DEBUG_message( "%u, ", data->address );
      DEBUG_message( "%u, ", data->command );
      DEBUG_message( "%u, ", data->flags );
      DEBUG_message( "%u, ", u16_ir_timer );
      DEBUG_message( "\r\n" );

      //first code
      if(ir_index_mem == 0xff && u16_ir_timer == 0xffff)
      {
        ir_index_mem = 0;
        ir_data_mem = &irmp_data_mem[ir_index_mem];
        ir_time_mem = &u16_time_stored[ir_index_mem];
        *ir_time_mem = 0xffff;
        *ir_data_mem = *data;
        ir_repeat_counter = 0;
        if( ir_index_mem >= (MAX_STORAGE_PER_CH-1)) //stop if index exceeds limit
        {
          ir_state = LEARNING_END;
        }
      }
      else
      {
        if( (data->flags & IRMP_FLAG_REPETITION 
             || (data->address == irmp_data_mem[ir_index_mem].address && data->protocol == irmp_data_mem[ir_index_mem].protocol) ) 
           && u16_ir_timer < 100 && ir_repeat_counter < IRSND_MAX_REPETITIONS)
        {
          ir_repeat_counter++;
        }
        else
        {
          //save time and repeat_counter, prepare next command
          ir_data_mem->flags = ir_repeat_counter;
          ir_repeat_counter = 0;
          *ir_time_mem = u16_ir_timer;
          ir_index_mem++;
          ir_data_mem = &irmp_data_mem[ir_index_mem];
          ir_time_mem = &u16_time_stored[ir_index_mem];
          *ir_data_mem = *data;
          *ir_time_mem = 0xffff;
          if( ir_index_mem >= (MAX_STORAGE_PER_CH-1)) //stop if index exceeds limit
          {
            ir_state = LEARNING_END;
          }
        }
      }
      u16_ir_timer = 0; //reset time between codes
    }
  }
  else if(ir_state == LEARNING_END)
  {
    if(ir_index_mem != 0xff)
    {
      ir_data_mem->flags = ir_repeat_counter;
      //write data to EEprom
      for(uint8_t i = 0; i <= ir_index_mem; i++)
      {
        MY_FUNCTIONS_EEPROM_save_ir(u8_ir_act_channel, i, irmp_data_mem[i], u16_time_stored[i]);
      }
    }
    
    //select off command for timer channels
    if(MY_FUNCTIONS_IR_get_act_channel() == S9)
    {
      MY_FUNCTIONS_IR_start_learning( S9+2 );
      MY_FUNCTIONS_LED_set(LED_PULSE_ORANGE, __TRUE);
      TIMER_LED1_ON;
    }
    else if(MY_FUNCTIONS_IR_get_act_channel() == S10)
    {
      MY_FUNCTIONS_IR_start_learning( S10+2 );
      MY_FUNCTIONS_LED_set(LED_PULSE_ORANGE, __TRUE);
      TIMER_LED2_ON;
    }
    else
    {
      ir_state = LEARNING_OFF;
      MY_FUNCTIONS_LED_set(LED_OFF, __TRUE);
      
      //restore timer LED state
      if( MY_FUNCTIONS_TIMER_tim1_is_active() )
      {
        if( MY_FUNCTIONS_TIMER_tim1_get_laststate() != __TRUE )
        {
          TIMER_LED1_FADING;
        }
        else
        {
          TIMER_LED1_ON;
        }
      }
      else
      {
        TIMER_LED1_OFF;
      }
      if( MY_FUNCTIONS_TIMER_tim2_is_active() )
      {
        if( MY_FUNCTIONS_TIMER_tim2_get_laststate() != __TRUE )
        {
          TIMER_LED2_FADING;
        }
        else
        {
          TIMER_LED2_ON;
        }
      }
      else
      {
        TIMER_LED2_OFF;
      }
    }
  }
  else if(ir_state == LEARNING_OFF)
  {
    MY_FUNCTIONS_TIMER_do(); //execute timerflags only if IR not active
    if( irmp_get_data (data))
    {
      //blink green
      MY_FUNCTIONS_LED_set(LED_PULSE_GREEN, __TRUE);
      DEBUG_message( "IRMP: ");
      DEBUG_message( "%u, ", data->protocol );
      DEBUG_message( "%u, ", data->address );
      DEBUG_message( "%u, ", data->command );
      DEBUG_message( "%u, ", data->flags );
      DEBUG_message( "%u, ", u16_ir_timer );
      DEBUG_message( "\r\n" );
    }
  }
}

/* @brief timer for ir handling call every 10ms
 *
 * @param none 
 * @return none
**/
void MY_FUNCTIONS_IR_timer( void )
{
  if(ir_state == LEARNING_ON)
  {
    if( u16_ir_timer != 0xffffu )
    {
      u16_ir_timer++;
    }
  }
  if(ir_state == IR_SEND)
  {
    if( u16_ir_timer != 0xffffu && u16_ir_timer != 0u)
    {
      u16_ir_timer--;
    }
  }
}

/* @brief get actual ir state
 *
 * @param none 
 * @return ir_state
**/
IR_state_t MY_FUNCTIONS_IR_get_state( void )
{
  return ir_state;
}

/* @brief get actual channel
 *
 * @param none 
 * @return u8_ir_act_channel
**/
uint8_t MY_FUNCTIONS_IR_get_act_channel( void )
{
  return u8_ir_act_channel;
}

/* @brief initialises learning new ir commands for a channel
 *
 * @param channel - channel to store new ir data 
 * @return none
**/
void MY_FUNCTIONS_IR_start_learning( uint8_t channel )
{
  if(channel == 0xff)
  {
    ir_state = LEARNING_SELECT_CH;
    u8_ir_act_channel = 0xff;
    //turn TIMER LEDs off during learning mode
    TIMER_LED1_OFF;
    TIMER_LED2_OFF;
  }
  else
  {
    u8_ir_act_channel = channel;
    ir_state = LEARNING_ON;
    u16_ir_timer = U16_TIMER_DISABLE;
    ir_index_mem = 0xff;
  }
}

/* @brief stop learning /select off channel
 * @param none
 * @return none
**/
void MY_FUNCTIONS_IR_stop_learning( void )
{
  ir_state = LEARNING_END;
}

/* @brief initialises ir sending for a channel
 *
 * @param channel - channel for ir data to be send 
 * @return none
**/
void MY_FUNCTIONS_IR_send( uint8_t channel )
{
  ir_index_mem = 0;
  ir_state = IR_SEND;
  u16_ir_timer = 0;
  u8_ir_act_channel = channel;
}

/* @brief abort actual ir sending
 * @param none
 * @return none
**/
void MY_FUNCTIONS_IR_abort_sending( void )
{
  ir_index_mem = 0xff;
  ir_state = LEARNING_OFF;
  u16_ir_timer = U16_TIMER_DISABLE;
  u8_ir_act_channel = 0xff;
  MY_FUNCTIONS_LED_set(LED_OFF, __TRUE);
}

/******************************************************************************
 *  MY_FUNCTIONS_LED functions
 *  this sections containt the functions for duo color device led
******************************************************************************/
/* @brief do function for duo color led call every 100ms
 * @note changes led state and creates the led timings
 * @param none
 * @return none
**/
void MY_FUNCTIONS_LED_do()
{
  if(u8_led_time == 0)
  {
    switch(led_state)
    {
      case LED_RED:
        GPIO_ODR(LED_GREEN_PORT) &= (uint8_t)(~LED_GREEN_PIN);
        GPIO_ODR(LED_RED_PORT) |= (LED_RED_PIN);
        u8_led_time = U8_TIMER_DISABLE;
        break;
      case LED_GREEN:
        GPIO_ODR(LED_GREEN_PORT) |= (LED_GREEN_PIN);
        GPIO_ODR(LED_RED_PORT) &= (uint8_t)(~LED_RED_PIN);
        u8_led_time = U8_TIMER_DISABLE;
        break;
      case LED_ORANGE:
        GPIO_ODR(LED_GREEN_PORT) |= (LED_GREEN_PIN);
        GPIO_ODR(LED_RED_PORT) |= (LED_RED_PIN);
        u8_led_time = U8_TIMER_DISABLE;
        break;
      case LED_OFF:
        GPIO_ODR(LED_GREEN_PORT) &= (uint8_t)(~LED_GREEN_PIN);
        GPIO_ODR(LED_RED_PORT) &= (uint8_t)(~LED_RED_PIN);
        u8_led_time = U8_TIMER_DISABLE;
        break;
      case LED_PULSE_RED:
        GPIO_ODR(LED_GREEN_PORT) &= (uint8_t)(~LED_GREEN_PIN);
        GPIO_ODR(LED_RED_PORT) |= (LED_RED_PIN);
        u8_led_time = 2;
        led_state = LED_PULSE_OFF;
        break;
      case LED_PULSE_GREEN:
        GPIO_ODR(LED_GREEN_PORT) |= (LED_GREEN_PIN);
        GPIO_ODR(LED_RED_PORT) &= (uint8_t)(~LED_RED_PIN);
        u8_led_time = 2;
        led_state = LED_PULSE_OFF;
        break;
      case LED_PULSE_ORANGE:
        GPIO_ODR(LED_GREEN_PORT) |= (LED_GREEN_PIN);
        GPIO_ODR(LED_RED_PORT) |= (LED_RED_PIN);
        u8_led_time = 2;
        led_state = LED_PULSE_OFF;
        break;
     case LED_PULSE_OFF:
        GPIO_ODR(LED_GREEN_PORT) &= (uint8_t)(~LED_GREEN_PIN);
        GPIO_ODR(LED_RED_PORT) &= (uint8_t)(~LED_RED_PIN);
        u8_led_time = 8;
        led_state = LED_OFF;
        break;
     case LED_PULSE_OFF_RED:
        GPIO_ODR(LED_GREEN_PORT) &= (uint8_t)(~LED_GREEN_PIN);
        GPIO_ODR(LED_RED_PORT) &= (uint8_t)(~LED_RED_PIN);
        u8_led_time = 8;
        led_state = LED_RED;
        break;
    }
  }
  else if(u8_led_time != U8_TIMER_DISABLE)
  {
    u8_led_time--;
  }
}

/* @brief get led state of duo color led
 *
 * @param none
 * @return state of duo color led 
**/
led_state_t MY_FUNCTIONS_LED_get_state( void)
{
  return led_state;
}

/* @brief Change led state of duo color led
 *
 * @param state - new led state
 * @param immediatly - change immediatly if __TRUE
 * @return none
**/
void MY_FUNCTIONS_LED_set(led_state_t state, bool_t immediatly)
{
  led_state = state;
  if(immediatly)
  {
    u8_led_time = 0;
    MY_FUNCTIONS_LED_do();
  }
}


/******************************************************************************
 *  MY_FUNCTIOS_TIMER functions
 *  this sections containt the functions for timer channels
******************************************************************************/
/* @brief do function for timer
 * @param none
 * @return none
**/
void MY_FUNCTIONS_TIMER_do( void )
{
  if(led_timer1.flags & TIMER_ENABLE)
  {
    if(led_timer1.flags & TIMER_START)
    {
      TIMER_LED1_ON;
      led_timer1.flags &= ~TIMER_START;
      //load STOP_Time
      if( CONFIG_LOOKUP[ GET_TIM1_STOP_CONF ] == 0)
      {
        led_timer1.u32_timer_stop = U32_TIMER_DISABLE;
        TIMER_LED1_FADING;
      }
      else
      {
        led_timer1.u32_timer_stop = (uint32_t)CONFIG_LOOKUP[ GET_TIM1_STOP_CONF ]*60UL*60UL;
      }
      
      if( MY_FUNCTIONS_TIMER_tim1_get_laststate() != __TRUE )
      {
        MY_FUNCTIONS_IR_send( S9 );
        MY_FUNCTIONS_TIMER_tim1_set_laststate( __TRUE );
      }
    }
    if(led_timer1.flags & TIMER_STOP)
    {
      TIMER_LED1_FADING;
      led_timer1.flags &= ~TIMER_STOP;
      
      if( MY_FUNCTIONS_TIMER_tim1_get_laststate() == __TRUE )
      {
        MY_FUNCTIONS_IR_send( S9+2 );
        MY_FUNCTIONS_TIMER_tim1_set_laststate( __FALSE );
      }
    }
  }
  if(led_timer2.flags & TIMER_ENABLE)
  {
    if(led_timer2.flags & TIMER_START)
    {
      TIMER_LED2_ON;
      led_timer2.flags &= ~TIMER_START;
      //load STOP_Time
      if( CONFIG_LOOKUP[ GET_TIM2_STOP_CONF ] == 0)
      {
        led_timer2.u32_timer_stop = U32_TIMER_DISABLE;
        TIMER_LED2_FADING;
      }
      else
      {
        led_timer2.u32_timer_stop = (uint32_t)CONFIG_LOOKUP[ GET_TIM2_STOP_CONF ]*60UL*60UL;
      }
      
      if( MY_FUNCTIONS_TIMER_tim2_get_laststate() != __TRUE )
      {
        MY_FUNCTIONS_IR_send( S10 );
        MY_FUNCTIONS_TIMER_tim2_set_laststate( __TRUE );
      }
    }
    if(led_timer2.flags & TIMER_STOP)
    {
      TIMER_LED2_FADING;
      led_timer2.flags &= ~TIMER_STOP;
            
      if( MY_FUNCTIONS_TIMER_tim2_get_laststate() == __TRUE )
      {
        MY_FUNCTIONS_IR_send( S10+2 );
        MY_FUNCTIONS_TIMER_tim2_set_laststate( __FALSE );
      }
    }
  }
}

/* @brief timer for the timer channels call every 1s
 * @note should be called from isr to avoid false timings
 * @param none
 * @return none
**/
void MY_FUNCTIONS_TIMER_isr( void )
{
  if(led_timer1.flags & TIMER_ENABLE)
  {
    if(led_timer1.u32_timer == 0)
    {
      led_timer1.flags |= TIMER_START;
    }
    else if(led_timer1.u32_timer == led_timer1.u32_timer_stop)
    {
      led_timer1.flags |= TIMER_STOP;
    }
  }
  if(led_timer1.u32_timer != U32_TIMER_DISABLE)
  {
    led_timer1.u32_timer++;
    if(led_timer1.u32_timer >= 24UL*60UL*60UL)
    {
      led_timer1.u32_timer = 0;
    }
  }
  if(led_timer2.flags & TIMER_ENABLE)
  {
    if(led_timer2.u32_timer == 0)
    {
      led_timer2.flags |= TIMER_START;
    }
    else if(led_timer2.u32_timer == led_timer2.u32_timer_stop)
    {
      led_timer2.flags |= TIMER_STOP;
    }
  }
  if(led_timer2.u32_timer != U32_TIMER_DISABLE)
  {
    led_timer2.u32_timer++;
    if(led_timer2.u32_timer >= 24UL*60UL*60UL)
    {
      led_timer2.u32_timer = 0;
    }
  }
}

/* @brief save timer 1 state
 * @param new_state of timer 1
 * @return none
**/
void MY_FUNCTIONS_TIMER_tim1_set_laststate( bool_t new_state )
{
  led_timer1.last_state = new_state;
}

/* @brief save timer 2 state
 * @param new_state of timer 2
 * @return none
**/
void MY_FUNCTIONS_TIMER_tim2_set_laststate( bool_t new_state )
{
  led_timer2.last_state = new_state;
}

/* @brief get fucntion for last timer 1 state
 * @param none 
 * @return state of timer 1
**/
bool_t MY_FUNCTIONS_TIMER_tim1_get_laststate( void )
{
  return led_timer1.last_state;
}

/* @brief get fucntion for last timer 3 state
 * @param none 
 * @return state of timer 3
**/
bool_t MY_FUNCTIONS_TIMER_tim2_get_laststate( void )
{
  return led_timer2.last_state;
}

/* @brief enable and restart timer 1
 * @param none 
 * @return none
**/
void MY_FUNCTIONS_TIMER_tim1_restart( void )
{
  led_timer1.u32_timer = 0;
  led_timer1.flags |= TIMER_ENABLE;
}

/* @brief enable and restart timer 2
 * @param none 
 * @return none
**/
void MY_FUNCTIONS_TIMER_tim2_restart()
{
  led_timer2.u32_timer = 0;
  led_timer2.flags |= TIMER_ENABLE;
}

/* @brief disable and stop timer 1
 * @param none 
 * @return none
**/
void MY_FUNCTIONS_TIMER_tim1_stop( void )
{
  led_timer1.u32_timer = 0;
  led_timer1.flags = TIMER_DISABLE;
}

/* @brief disable and stop timer 2
 * @param none 
 * @return none
**/void MY_FUNCTIONS_TIMER_tim2_stop( void )
{
  led_timer2.u32_timer = 0;
  led_timer2.flags = TIMER_DISABLE;
}

/* @brief get function for timer 1 activation state
 * @param none 
 * @return timer 1 activation state
**/
bool_t MY_FUNCTIONS_TIMER_tim1_is_active( void )
{
  return (bool_t)(led_timer1.flags & TIMER_ENABLE);
}

/* @brief get function for timer 2 activation state
 * @param none 
 * @return timer 2 activation state
**/
bool_t MY_FUNCTIONS_TIMER_tim2_is_active( void )
{
  return (bool_t)(led_timer2.flags & TIMER_ENABLE);
}

/* @brief Init funtion for timer LEDs
 * @param none 
 * @return none
**/
void MY_FUNCTIONS_TIMER_LEDS_init( void )
{
  TIM3_CR1  = 0x00;
  TIM3_CR2  = 0x00;
  TIM3_SMCR = 0x00;
  TIM3_ETR  = 0x00;
  TIM3_IER  = 0x00;

  /* Disable channels */
  TIM3_CCER1 = 0x00;
  /* Configure channels as inputs: it is necessary if lock level is equal to 2 or 3 */
  TIM3_CCMR1 = 0x01;
  TIM3_CCMR2 = 0x01;

  /* Then reset channel registers: it also works if lock level is equal to 2 or 3 */
  TIM3_CCER1 = 0x00;
  TIM3_CCMR1 = 0x00;
  TIM3_CCMR2 = 0x00;
  TIM3_CNTRH = 0x00;
  TIM3_CNTRL = 0x00;
  TIM3_ARRH  = 0xFF;
  TIM3_ARRL  = 0xFF;
  TIM3_CCR1H = 0x00;
  TIM3_CCR1L = 0x00;
  TIM3_CCR2H = 0x00;
  TIM3_CCR2L = 0x00;
  TIM3_OISR  = 0x00;
  TIM3_EGR   = TIM_EGR_UG;
  TIM3_BKR   = 0x00;
  TIM3_SR1   = 0x00;
  TIM3_SR2   = 0x00;
  
  clk_periph_clock_enable(CLK_TIM3);
  
  timer_set_mode (TIM3_BASE, TIM_CMS_CENTER_1,TIM_DIR_UP);
  timer_set_prescaler(TIM3_BASE, 8-1);
  timer_set_period(TIM3_BASE, 255);
  timer_set_repetition_counter( TIM3_BASE, 0);

  timer_enable_oc_output(TIM3_BASE, TIM_OC1);
  timer_set_oc_mode(TIM3_BASE, TIM_OC1, TIM_OCM_PWM_1);
  timer_enable_oc_output(TIM3_BASE, TIM_OC2);
  timer_set_oc_mode(TIM3_BASE, TIM_OC2, TIM_OCM_PWM_1);

  /* Set the Pulse value */
  timer_set_oc_value(TIM3_BASE, TIM_OC1, 0);
  timer_set_oc_value(TIM3_BASE, TIM_OC2, 0);
  
  timer_enable_oc_preload(TIM3_BASE, TIM_OC1);
  timer_enable_oc_preload(TIM3_BASE, TIM_OC2);
  
  timer_enable_counter(TIM3_BASE);
  timer_enable_break_main_output(TIM3_BASE);
}

/* @brief Timer LED fading function
 * @note should be called every 10ms
 * @param none 
 * @return none
**/
void MY_FUNCTIONS_TIMER_LEDS_fading( void )
{
  static uint8_t fadingspeed = 0;
  static uint8_t fadingvalue = 0;
  
  if(fadingspeed++ > 1)
  {
    fadingspeed = 0;
    fadingvalue++;
    if(fadingvalue==254)
    {
      fadingvalue = 0;
    }
    
    if(led_timer1.flags & TIMER_ENABLE )
    {
      if( led_timer1.fading != __FALSE )
      {
        timer_set_oc_value(TIM3_BASE,TIM_OC1, FADING_TABLE[fadingvalue] );
      }
    }
    
    if(led_timer2.flags & TIMER_ENABLE)
    {
      if( led_timer2.fading != __FALSE )
      {
        timer_set_oc_value(TIM3_BASE,TIM_OC2, FADING_TABLE[fadingvalue] );
      }
    }
  }
}


/******************************************************************************
 *  MY_FUNCTIOS_EEPROM functions
 *  this sections containt the functions for data storage in eeprom
******************************************************************************/
/* @brief Init EEPROM data
 * @param none
 * @return none
**/
void MY_FUNCTIONS_EEPROM_init( void )
{
  IRMP_DATA empty_irmp = { 0 };
  for(uint8_t i = 0; i < (S10+3)*MAX_STORAGE_PER_CH; i++ )
  {
    MY_FUNCTIONS_EEPROM_set_irmp_code( empty_irmp, i );
    MY_FUNCTIONS_EEPROM_set_ir_time( U16_TIMER_DISABLE, i );
  }
  /* Set the EEPROM variable indicating that the initialization was successful */
  MY_FUNCTIONS_EEPROM_set_init_value( EEPROM_INIT_VALUE );
}

/**
 * @brief save ir code and time in eeprom
 * @param channel - channel of variable
 * @param index - index of variable
 * @param data - ir data
 * @param time - ir time
 * @return none
**/
void MY_FUNCTIONS_EEPROM_save_ir(uint8_t channel, uint8_t index, IRMP_DATA data, uint16_t time)
{
  uint8_t memory_index = channel*MAX_STORAGE_PER_CH+index;
  MY_FUNCTIONS_EEPROM_set_irmp_code( data, memory_index );
  MY_FUNCTIONS_EEPROM_set_ir_time( time, memory_index );
}

/**
 * @brief get the saved ir code and time  from eeprom
 * @param channel - channel of variable
 * @param index - index of variable
 * @param data - pointer to ir data storage
 * @param time - pointer to ir time storage
**/
void MY_FUNCTIONS_EEPROM_get_ir(uint8_t channel, uint8_t index, IRMP_DATA* data, uint16_t* time)
{
  uint8_t memory_index = channel*MAX_STORAGE_PER_CH+index;
  *data = MY_FUNCTIONS_EEPROM_get_irmp_code( memory_index );
  *time = MY_FUNCTIONS_EEPROM_get_ir_time( memory_index );
}

/**
 * @brief returns the value of the ee_irmp_code[X] variable
 * @param index of value
 * @return value of ee_irmp_code[X]
**/
IRMP_DATA MY_FUNCTIONS_EEPROM_get_irmp_code( uint8_t index )
{
  return ee_irmp_code[index];
}

/**
 * @brief set a value of the ee_irmp_code[X] variable
 * @param IRMP_DATA new_value - new value of ee_irmp_code[X] variable
 * @param uint8_t index - index of ee_irmp_code[X] variable
 * @return none
**/
void MY_FUNCTIONS_EEPROM_set_irmp_code( IRMP_DATA new_value, uint8_t index )
{
  /* Unlocks the data EEPROM memory */
  FLASH_DUKR = FLASH_DUKR_1;
  FLASH_DUKR = FLASH_DUKR_2;
  /* wait while data EEPROM memory is not unlocked */
  while (!(FLASH_IAPSR&FLASH_IAPSR_DUL));
  /* write data to EEPROM variable */
  *((__near IRMP_DATA*)&ee_irmp_code[index]) = *((IRMP_DATA*)&(new_value));
  /* Waits for a Flash operation to complete */
  uint16_t timeout = 0xffff;
  while(((uint8_t)(FLASH_IAPSR & (uint8_t)(FLASH_IAPSR_HVOFF | FLASH_IAPSR_WR_PG_DIS)) == 0x00) && (timeout != 0x00))
  {
    timeout--;
  }
  /* Locks the program or data EEPROM memory */
  FLASH_IAPSR &= 0xf7;
}

/**
 * @brief returns a value of the ee_irmp_time[X] variable
 * @param index of value
 * @return value of ee_irmp_time[X]
**/
uint16_t MY_FUNCTIONS_EEPROM_get_ir_time( uint8_t index )
{
  return ee_irmp_time[index];
}

/**
 * @brief set the value of the ee_irmp_time[X] variable
 * @param uint16_t new_value - new value of ee_irmp_time[X] variable
 * @param uint8_t index - index of ee_irmp_time[X] variable
 * @return none
**/
void MY_FUNCTIONS_EEPROM_set_ir_time( uint16_t new_value, uint8_t index )
{
  /* Unlocks the data EEPROM memory */
  FLASH_DUKR = FLASH_DUKR_1;
  FLASH_DUKR = FLASH_DUKR_2;
  /* wait while data EEPROM memory is not unlocked */
  while (!(FLASH_IAPSR&FLASH_IAPSR_DUL));
  /* write data to EEPROM variable */
  *((__near uint16_t*)&ee_irmp_time[index]) = *((uint16_t*)&(new_value));
  /* Waits for a Flash operation to complete */
  uint16_t timeout = 0xffff;
  while(((uint8_t)(FLASH_IAPSR & (uint8_t)(FLASH_IAPSR_HVOFF | FLASH_IAPSR_WR_PG_DIS)) == 0x00) && (timeout != 0x00))
  {
    timeout--;
  }
  /* Locks the program or data EEPROM memory */
  FLASH_IAPSR &= 0xf7;
}

/**
 * @brief returns the value of the ee_u32_init variable
 * @param none
 * @return value of ee_u32_init
**/
uint32_t MY_FUNCTIONS_EEPROM_get_init_value( void )
{
  return ee_u32_init;
}

/**
 * @brief set the value of the ee_u32_init variable
 * @param uint32_t u32_new_init_value - new value of ee_u32_init variable
 * @return none
**/
void MY_FUNCTIONS_EEPROM_set_init_value( uint32_t u32_new_init_value )
{
  /* Unlocks the data EEPROM memory */
  FLASH_DUKR = FLASH_DUKR_1;
  FLASH_DUKR = FLASH_DUKR_2;
  /* wait while data EEPROM memory is not unlocked */
 while (!(FLASH_IAPSR&FLASH_IAPSR_DUL));
  /* write data to EEPROM variable */
  *((__near uint32_t*)&ee_u32_init) = *((uint32_t*)&(u32_new_init_value));
  /* Waits for a Flash operation to complete */
  uint16_t timeout = 0xffff;
  while(((uint8_t)(FLASH_IAPSR & (uint8_t)(FLASH_IAPSR_HVOFF | FLASH_IAPSR_WR_PG_DIS)) == 0x00) && (timeout != 0x00))
  {
    timeout--;
  }
  /* Locks the program or data EEPROM memory */
  FLASH_IAPSR &= 0xf7;
}
/** @}
 * End of file
**/