/**
 * @file    my_functions.h
 * @brief   This file contains functionprototypes for the function collection
 *          for IR, LED, TIMER and EEPROM handling
 * @author  ELV Elektronik AG
**/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2021-2022 ELV Elektronik AG - info@elv.com
 * 
 * This file is part of ELV-SH-IRS8 Firmware.
 * ELV-SH-IRS8 Firmware is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ELV-SH-IRS8 Firmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with ELV-SH-IRS8 Firmware. If not, see <https://www.gnu.org/licenses/>. 
 *---------------------------------------------------------------------------*/
#ifndef __MY_FUNCTIONS_H__
#define __MY_FUNCTIONS_H__

/*
 *  Includes ------------------------------------------------------------------
 */
#include "typedefs_stm8.h"
#include "irmp.h"

/*
 *  Definitions ---------------------------------------------------------------
 */
#define U8_TIMER_DISABLE 0xff
#define U16_TIMER_DISABLE 0xffff 
#define U32_TIMER_DISABLE 0xffffffff
/******************************************************************************
 *  MY_FUNCTIONS_IR definitions
 *  this section containt the definitions for the ir handling
******************************************************************************/
typedef enum
{
  LEARNING_OFF,
  LEARNING_SELECT_CH,
  LEARNING_ON,
  LEARNING_END,
  IR_SEND,
}IR_state_t;

/******************************************************************************
 *  MY_FUNCTIONS_LED definitions
 *  this sections containt the definitions for duo color device led
******************************************************************************/
typedef enum
{
  LED_GREEN,
  LED_RED,
  LED_ORANGE,
  LED_PULSE_ORANGE,
  LED_PULSE_GREEN,
  LED_PULSE_RED,
  LED_PULSE_OFF,
  LED_PULSE_OFF_RED,
  LED_OFF,
}led_state_t;

/******************************************************************************
 *  MY_FUNCTIOS_TIMER definitions
 *  this sections containt the definitions and variables for timer channels
******************************************************************************/
typedef enum
{
  TIMER_DISABLE = 0,
  TIMER_ENABLE  = (1<<0),
  TIMER_START   = (1<<1),
  TIMER_STOP    = (1<<2),
}timer_flags_t;
typedef struct TIMER
{
  timer_flags_t flags;
  uint32_t u32_timer;
  uint32_t u32_timer_stop;
  bool_t fading;
  bool_t last_state;
}TIMER_t;
extern TIMER_t led_timer1;
extern TIMER_t led_timer2;

#define TIMER_LED1_ON     {timer_set_oc_value(TIM3_BASE,TIM_OC1, 180 );led_timer1.fading = __FALSE;}
#define TIMER_LED1_LOWON  {timer_set_oc_value(TIM3_BASE,TIM_OC1,  20 );led_timer1.fading = __FALSE;}
#define TIMER_LED1_OFF    {timer_set_oc_value(TIM3_BASE,TIM_OC1,   0 );led_timer1.fading = __FALSE;}
#define TIMER_LED1_FADING {timer_set_oc_value(TIM3_BASE,TIM_OC1,   0 );led_timer1.fading = __TRUE;}
#define TIMER_LED2_ON     {timer_set_oc_value(TIM3_BASE,TIM_OC2, 180 );led_timer2.fading = __FALSE;}
#define TIMER_LED2_LOWON  {timer_set_oc_value(TIM3_BASE,TIM_OC2,  20 );led_timer2.fading = __FALSE;}
#define TIMER_LED2_OFF    {timer_set_oc_value(TIM3_BASE,TIM_OC2,   0 );led_timer2.fading = __FALSE;}
#define TIMER_LED2_FADING {timer_set_oc_value(TIM3_BASE,TIM_OC2,   0 );led_timer2.fading = __TRUE;}

/******************************************************************************
 *  MY_FUNCTIOS_EEPROM definitions
 *  this sections containt the definitions for data storage in eeprom
******************************************************************************/
#define FACTORY_RESET_VALUE 0x00000000
#define EEPROM_INIT_VALUE   0xbecaffee

/*
 *  Prototypes ----------------------------------------------------------------
 */
/******************************************************************************
 *  MY_FUNCTIONS_IR prototypes
 *  this section containt the prototypes for the ir handling
******************************************************************************/
extern void MY_FUNCTIONS_IR_init( void );
extern void MY_FUNCTIONS_IR_timer( void );
extern void MY_FUNCTIONS_IR_do( void );
extern void MY_FUNCTIONS_IR_send( uint8_t channel );
extern void MY_FUNCTIONS_IR_abort_sending( void );
extern IR_state_t MY_FUNCTIONS_IR_get_state( void );
extern uint8_t MY_FUNCTIONS_IR_get_act_channel( void );
extern void MY_FUNCTIONS_IR_stop_learning( void );
extern void MY_FUNCTIONS_IR_start_learning( uint8_t channel );

/******************************************************************************
 *  MY_FUNCTIONS_LED prototypes
 *  this sections containt the prototypes for duo color device led
******************************************************************************/
extern void MY_FUNCTIONS_LED_do( void );
extern led_state_t MY_FUNCTIONS_LED_get_state( void);
extern void MY_FUNCTIONS_LED_set(led_state_t state, bool_t immediatly);

/******************************************************************************
 *  MY_FUNCTIOS_TIMER prototypes
 *  this sections containt the prototypes for timer channels
******************************************************************************/
extern void MY_FUNCTIONS_TIMER_do( void );
extern void MY_FUNCTIONS_TIMER_isr( void );
extern void MY_FUNCTIONS_TIMER_tim1_set_laststate( bool_t new_state );
extern void MY_FUNCTIONS_TIMER_tim2_set_laststate( bool_t new_state );
extern bool_t MY_FUNCTIONS_TIMER_tim1_get_laststate( void );
extern bool_t MY_FUNCTIONS_TIMER_tim2_get_laststate( void );
extern void MY_FUNCTIONS_TIMER_tim1_restart( void );
extern void MY_FUNCTIONS_TIMER_tim2_restart( void );
extern void MY_FUNCTIONS_TIMER_tim1_stop( void );
extern void MY_FUNCTIONS_TIMER_tim2_stop( void );
extern bool_t MY_FUNCTIONS_TIMER_tim1_is_active( void );
extern bool_t MY_FUNCTIONS_TIMER_tim2_is_active( void );
extern void MY_FUNCTIONS_TIMER_LEDS_init( void );
extern void MY_FUNCTIONS_TIMER_LEDS_fading( void );

/******************************************************************************
 *  MY_FUNCTIOS_EEPROM prototypes
 *  this sections containt the prototypes for data storage in eeprom
******************************************************************************/
extern void MY_FUNCTIONS_EEPROM_init( void );
extern void MY_FUNCTIONS_EEPROM_get_ir(uint8_t channel, uint8_t index, IRMP_DATA* data, uint16_t* time);
extern void MY_FUNCTIONS_EEPROM_save_ir(uint8_t channel, uint8_t index, IRMP_DATA data, uint16_t time);
extern IRMP_DATA MY_FUNCTIONS_EEPROM_get_irmp_code( uint8_t index );
extern void MY_FUNCTIONS_EEPROM_set_irmp_code( IRMP_DATA new_value, uint8_t index );
extern uint16_t MY_FUNCTIONS_EEPROM_get_ir_time( uint8_t index );
extern void MY_FUNCTIONS_EEPROM_set_ir_time( uint16_t new_value, uint8_t index );
extern uint32_t MY_FUNCTIONS_EEPROM_get_init_value( void );
extern void MY_FUNCTIONS_EEPROM_set_init_value( uint32_t u32_new_init_value );
extern uint32_t MY_FUNCTIONS_EEPROM_get_bootloader_value( void );
extern void MY_FUNCTIONS_EEPROM_set_bootloader_value( uint32_t u32_new_bootloader_value );

#endif /* __MY_FUNCTIONS_H__ */
/** @}
 * End of file
**/
