/**
 * @file    typedefs_stm8.h
 * @brief   This file contains the type definitions for the 
 *          IAR Embedded Workbench IDE for STM8 mikrocontroler
 *          in consideration to the firmware style guide.<br><br>
 *          Include after the stm8l15x.h! 
 * @author  ELV Elektronik AG
**/
/*-----------------------------------------------------------------------------
 * Copyright (c) 2021-2022 ELV Elektronik AG - info@elv.com
 * 
 * This file is part of ELV-SH-IRS8 Firmware.
 * ELV-SH-IRS8 Firmware is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ELV-SH-IRS8 Firmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with ELV-SH-IRS8 Firmware. If not, see <https://www.gnu.org/licenses/>. 
 *---------------------------------------------------------------------------*/
#ifndef __TYPEDEFS_STM8_H__
#define __TYPEDEFS_STM8_H__

/**
 * Integer types: unsigned
**/
#ifdef uint8_t 
typedef unsigned char uint8_t;        /*  8 bits, 0 to 255    */
#endif
#ifdef uint16_t
typedef unsigned short uint16_t;      /* 16 bits, 0 to 65535  */
#endif
#ifdef uint32_t
typedef unsigned long uint32_t;       /* 32 bits, 0 to 2^32-1 */
#endif
#ifdef uint64_t
typedef unsigned long long uint64_t;  /* 64 bits, 0 to 2^64-1 */
#endif

/** 
 * Integer types: signed
**/
typedef signed char sint8_t;				/*  8 bits, -128 to 127     */
typedef signed short sint16_t;				/* 16 bits, -32768 to 32767 */
typedef signed long sint32_t;				/* 32 bits, -2^31 to 2^31-1 */
typedef signed long long sint64_t;			/* 64 bits, -2^63 to 2^63-1 */

/**
 * Floating point types
**/
typedef float float32_t;					/* 1 bit sign,  8 bits exponent, 23 bits mantissa */
typedef long double float64_t;				/* 1 bit sign, 11 bits exponent, 52 bits mantissa */

/**
 * Bool type
**/
enum
{
  __FALSE = 0,
  __TRUE  = !__FALSE
};
typedef _Bool bool_t;


#endif /* __TYPEDEFS_STM8_H__ */
/** @}
 * End of file
**/
